@@ -1,6 +1,6 @@
 #!/home/chrisarg/perl5/perlbrew/perls/current/bin/perl
 package Bit::Set;
-$Bit::Set::VERSION = '0.02';
+$Bit::Set::VERSION = '0.03';
 use strict;
 use warnings;
 use FFI::Platypus;
@@ -337,7 +337,7 @@ Bit::Set - Perl interface for bitset functions from the 'bit' C library
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 SYNOPSIS
 
@@ -358,12 +358,12 @@ version 0.02
 
 =head1 DESCRIPTION
 
-This module provides a procedural Perl interface to the C library 'bit.h',
+This module provides a procedural Perl interface to the C library L<Bit|https://github.com/chrisarg/Bit>,
 for creating and manipulating bitsets. It uses C<FFI::Platypus> to wrap the
 C functions and C<Alien::Bit> to locate and link to the C library.
 
 The API is a direct mapping of the C functions. For detailed semantics of each
-function, please refer to the C<bit.h> header file documentation.
+function, please refer to L<Bit|https://github.com/chrisarg/Bit>.
 
 Runtime checks on arguments are performed if the C<DEBUG> environment variable
 is set to a true value when installing the code.
@@ -403,7 +403,7 @@ Loads an externally allocated bitset into a new Bit_T structure.
 =item B<Bit_extract(set, buffer)>
 
 Extracts the bitset from a Bit_T into an externally allocated buffer.
-Look at EXAMPLES for usage of the load and extract functions using FFI::Platypus.
+Look at EXAMPLES for usage of the load and extract functions using C<FFI::Platypus>.
 
 =back
 
@@ -536,7 +536,7 @@ creating a new bitset.
 
 =head1 EXAMPLES
 
-Examples of the use of the Bit::Set module. Many of these examples are lifted 
+Examples of the use of the C<Bit::Set> module. Many of these examples are lifted 
 from the test suite.
 Others are Perl "translations" of the original C benchmarks.
 
@@ -547,7 +547,7 @@ Others are Perl "translations" of the original C benchmarks.
 Simple example in which we create, set and test for setting of individual 
 bits into a bitset.
 
-  use Bit::Set;
+  use Bit::Set qw(:all);
 
   my $bitset = Bit_new(64);
   Bit_set($bitset, 1);
@@ -563,7 +563,7 @@ bits into a bitset.
 =item Example 2: Comparison operations between bitsets
 
 This example illustrates the use of the comparison functions provided by the 
-Bit::Set module. The equality comparison function is shown for simplicity, but 
+C<Bit::Set> module. The equality comparison function is shown for simplicity, but 
 the example can serve as blue print for
 other comparisons functions e.g. less than equal to.
 
@@ -589,7 +589,7 @@ other comparisons functions e.g. less than equal to.
 =item Example 3: Set operations on bitsets
 
 This example demonstrates the use of set operations provided by the 
-Bit::Set module. In this example, we will form the union of two bitsets 
+C<Bit::Set> module. In this example, we will form the union of two bitsets 
 into a new bitset. Then we will make sure that the union bitset contains 
 all the bits from both original bitsets.
 
@@ -671,7 +671,7 @@ or advanced, SIMD accelerated algorithms for efficient population counting.
 =item Example 5: Loading and extracting a bitset
 
 A slightly more complex example, in which we create a bitset, set a few bits,
- extract them into a buffer (allocated via FFI::Platypus::Buffer, though other
+ extract them into a buffer (allocated via C<FFI::Platypus::Buffer>, though other
  possibilities exist e.g. through Task::MemManager) and then checking that their
  values is correct. The load example reverses the logic, i.e. we allocate the
  buffer, set its value using pack, put the buffer into a bitset and test the
@@ -733,7 +733,6 @@ application in C.
     use warnings;
     use Time::HiRes  qw(gettimeofday tv_interval);
     use Bit::Set     qw(:all);
-    use Bit::Set::DB qw(:all);
 
     # Constants
     use constant BPQW => 64;    # bits per qword (8 bytes * 8 bits)
@@ -884,8 +883,8 @@ markdown file. The prompt used was the following:
     relevant check involves ONLY DEBUG, otherwise the code may not be stripped.
     Things to adhere to during the implementation:
 
-    The functions for the Bit_T, should end up in the module Bit::Set, and those for 
-    Bit_DB to Bit::Set::DB .
+    The functions for the Bit_T, should end up in the module C<Bit::Set>, and those for 
+    Bit_DB to C<Bit::Set::DB> .
     1. Ensure that you implement the Perl interface to all the functions in the C 
     interface, i.e. don't implement some functions and then tell me the others are 
     implemented similarly! Reflect that you have implemented all the functions by 
@@ -904,20 +903,20 @@ markdown file. The prompt used was the following:
     4. When implementing a function, do provide the POD documentation for it. 
     However, generate the POD after you have implemented the functions.
     5. After you have implemented the modules, generate a simple test that will 
-    generate a Bit::Set of capacity of 1024 bits, set the first, second and 5th one 
+    generate a C<Bit::Set> of capacity of 1024 bits, set the first, second and 5th one 
     and see if the popcount is equal to 3.
 
 Claude did get *most* things right:
 
 =over 5
 
-=item * it generated 3 chunks of code corresponding to `Bit::Set`,  `Bit::Set::DB` and the single test file
+=item * it generated 3 chunks of code corresponding to C<Bit::Set>,  C<Bit::Set::DB> and the single test file
 
 =item * the table driven approach was implemented effectively reducing the number of lines of code that had to be written
 
-=item * The checked runtime exceptions in the C interface were incorporated in the Perl using a wrapper function that was provided to `FFI::Platypus` `attach`.
+=item * The checked runtime exceptions in the C interface were incorporated in the Perl using a wrapper function that was provided to C<FFI::Platypus> attach.
 
-=item * The `FFI::Platypus::Record` was correctly selected into the implementation for the C structure that passes options for the CPU/GPU enhanced container functions.
+=item * The C<FFI::Platypus::Record> was correctly selected into the implementation for the C structure that passes options for the CPU/GPU enhanced container functions.
 
 =item * the POD documentation was generated as a skeleton using the grouping of function in the README file. 
 The documentation was no frills, a very simple repetition of what is available from L<Bit|https://github.com/chrisarg/Bit>, but it is enough to
@@ -950,14 +949,14 @@ The relevant section is shown below and exhibits numerous problems.
 When the DEBUG variable is not set, it is unclear whether the check for DEBUG
 will strip the code that adds the runtime exception wrapper at compile time.
 The pattern discussed in the Perl documentation states that a simple test
-of the form `if (DEBUG) { ... }` will strip everything within the block, but
-will a test of the form `if ( DEBUG && exists $spec->{check} ) { ... }` do the 
-same? 
-Secondly, the attachment of the wrapper function to the FFI call is also a 
+of the form C< if (DEBUG) { ... }> will strip everything within the block, but
+will a test of the form C<< if ( DEBUG && exists $spec->{check} ) { ... } >> do the
+same?
+Secondly, the attachment of the wrapper function to the FFI call is also a
 concern: it takes place early in the process, before the DEBUG check is made.
-Thirdly, the snippet `push @attach_args, wrapper => sub { ... }` as it pushes
-*two* arguments into the function call for `attach`. 
-If one looks into the documentation for L<FFI::Platypus::attach|https://metacpan.org/pod/FFI::Platypus#attach>, 
+Thirdly, the snippet C<< push @attach_args, wrapper => sub { ... } >> as it pushes
+*two* arguments into the function call for C< attach >.
+If one looks into the documentation for L<FFI::Platypus::attach|https://metacpan.org/pod/FFI::Platypus#attach>,
 
     $ffi->attach($name => \@argument_types => $return_type);
     $ffi->attach([$c_name => $perl_name] => \@argument_types => $return_type);
@@ -967,9 +966,9 @@ If one looks into the documentation for L<FFI::Platypus::attach|https://metacpan
     $ffi->attach([$address => $perl_name] => \@argument_types => $return_type, \&wrapper);
 
 it becomes clear that the maintainer is using the fat comma instead of the
-regular comma to pass consecutive arguments into the `attach` function. 
+regular comma to pass consecutive arguments into the C<attach> function. 
 However, the chatbot is confusing the syntax and adding a hashlike key-value 
-pair when pushing the arguments of `attach`.
+pair when pushing the arguments of C<attach>.
 
 All these problems are reasonably easy to fix, by breaking the test involving 
 DEBUG into two nested ifs, moving the attach invocation at the end of the loop,
@@ -978,10 +977,10 @@ of the attach function.
 
 =item * The fat comma strikes again
 
-The container module (`Bit::Set::DB`) uses a C structure to pass options to the 
+The container module (C<Bit::Set::DB>) uses a C structure to pass options to the 
 CPU/hardware accelerator device . This C structure is passed by value and thus 
-should be passed as a `FFI::Platypus::Record`, created either as a separate 
-module file, or nested in the `Bit::Set::DB` module. The code that was actually
+should be passed as a C<FFI::Platypus::Record>, created either as a separate 
+module file, or nested in the C<Bit::Set::DB> module. The code that was actually
 generated by Claude looked like this:
 
     {
@@ -999,7 +998,7 @@ generated by Claude looked like this:
     }
 
 In the documentation for L<FFI::Platypus::Record|https://metacpan.org/pod/FFI::Platypus::Record#record_layout_1>, one can clearly see that the function record_layout_1 
-receives arguments as `record_layout_1($type => $name, ... );`, i.e. the fat comma 
+receives arguments as C<< record_layout_1($type => $name, ... ); >> , i.e. the fat comma 
 is used to separate consecutive arguments to the function, and not as part of the 
 definition of a hash. However Claude must "think" that it is dealing with a hash,
 as it reverses the order of the arguments to make the "keys" unique.
@@ -1016,12 +1015,49 @@ rather than the correct
 
     $ffi->type( 'record(Bit::Set::DB::SETOP_COUNT_OPTS)' => 'SETOP_COUNT_OPTS_t' )
 
+=item * Naive interpretation of returned pointers in the FFI interface
+
+A subtle LLM mistake concerns the handling of returned pointers from FFI calls. 
+A function in C that is declared as C<int* foo(...);> may use the returned
+pointer to provide a single value, or an array of values. Consider the proposal
+for C<BitDB_count> in the table driven interface:
+
+    BitDB_count => {
+        args  => ['Bit_DB_T'],
+        ret   => 'int*',
+        check => sub {
+            my ($set) = @_;
+            die "BitDB_count: set cannot be NULL" if !defined $set;
+        }
+    }
+
+When C<FFI::Platypus> encounters this return type, it will interpret the type
+as a hash reference to a Perl scalar as stated explicitly in the L<documentation|https://metacpan.org/pod/FFI::Platypus#Pointers>.
+The correct way to handle this is to declare the function as returning an L<opaque pointer|https://metacpan.org/pod/FFI::Platypus#Opaque-Pointers-(buffers-and-strings)>.
+In particular, one would rewrite the last snippet as:
+
+    BitDB_count => {
+        args  => ['Bit_DB_T'],
+        ret   => 'opaque',
+        check => sub {
+            my ($set) = @_;
+            die "BitDB_count: set cannot be NULL" if !defined $set;
+        }
+    }
+
+By doing so, one ends up receiving the memory address of the buffer as a Perl 
+scalar value, rather than a reference to Perl scalar, which must be dereferenced
+to yield the first (and only the first!) element of the array that is accessible
+through the pointer. Please refer to the documentation of L<FFI::Platypus|https://metacpan.org/pod/FFI::Platypus> for 
+more information on working with opaque pointers.
+The documentation of L<Bit::Set::DB|https://metacpan.org/pod/Bit::Set::DB> contains examples and usage patterns for 
+working with arrays returned from the Perl interface to C<Bit>.
 
 =back
 
 Having fixed these errors, I proceeded to generate a Perl version of the C
-test suite, by providing as context the (fixed) modules : `Bit::Set` and 
-`Bit::Set::DB` as well as the C source code for "test_bit.c". The actual
+test suite, by providing as context the (fixed) modules : C<Bit::Set> and 
+C<Bit::Set::DB> as well as the C source code for "test_bit.c". The actual
 prompt was the single liner:
 
     Convert this test file written in C to Perl, using the Bit::Set and Bit::Set::DB modules. 
@@ -1045,7 +1081,7 @@ following code is supposed to test the extraction of bits from a bitset:
 
 However, the code is utterly wrong (and segfaults!) as one has to provide
 the memory address of the buffer, not the Perl scalar value. The fix is to
-generate the buffer as a Perl string and then use `FFI::Platypus::Buffer` 
+generate the buffer as a Perl string and then use C<FFI::Platypus::Buffer> 
 to extract the memory address of the storage buffer used by the Perl scalar:
 
     my $scalar = "\0" x $buffer_size;    
